#include <WiFi.h>
#include <ESPmDNS.h>
#include <WiFiUdp.h>
#include <ArduinoOTA.h>
#include <Update.h>
#include <HTTPClient.h>
#include <Adafruit_NeoPixel.h>

const char* host = "<NameTheDevice>";
const char* ssid = "<WiFi-SSID>s";
const char* password = "<WiFi-Password>";

// Define the API endpoint
const char* apihost = "https://api.icmobile.singlewire.com/api/v1/notifications/";  // Replace with your API endpoint

const char* bearerToken = "<TokenFromInfromaCastApp>";

const int BUTTON_PIN = 9;
const int LED_PIN = 2;
const int NUM_LEDS = 1;
const int BUTTON_DELAY = 100; // delay time in milliseconds
const int BUTTON_TIMEOUT = 5000; // timeout duration in milliseconds

int buttonState = 0;
int lastButtonState = 0;
int buttonClicks = 0;
unsigned long buttonTimer = 0;

unsigned long lastRestartTime = 0;

Adafruit_NeoPixel leds = Adafruit_NeoPixel(NUM_LEDS, LED_PIN, NEO_GRB + NEO_KHZ800);


void setup() {
  Serial.begin(115200);
  Serial.println("Booting");


  Serial.begin(115200);

  pinMode(BUTTON_PIN, INPUT_PULLUP);

  leds.begin();
  leds.setPixelColor(0, leds.Color(0, 255, 0));
  leds.show();

  WiFi.onEvent(WiFiStationConnected, WiFiEvent_t::ARDUINO_EVENT_WIFI_STA_CONNECTED);
  WiFi.onEvent(WiFiGotIP, WiFiEvent_t::ARDUINO_EVENT_WIFI_STA_GOT_IP);
  WiFi.onEvent(WiFiStationDisconnected, WiFiEvent_t::ARDUINO_EVENT_WIFI_STA_DISCONNECTED);

  WiFi.mode(WIFI_STA);
  WiFi.begin(ssid, password);
  while (WiFi.waitForConnectResult() != WL_CONNECTED) {
    Serial.println("Connection Failed! Rebooting...");
    delay(5000);
    ESP.restart();
  }
  Serial.println("");
  Serial.print("Connected to ");
  Serial.println(ssid);
  Serial.print("Host: ");
  Serial.println(host);
  Serial.print("IP address: ");
  Serial.println(WiFi.localIP());
  Serial.print("MAC Address:  ");
  Serial.println(WiFi.macAddress());

  // Port defaults to 3232
  // ArduinoOTA.setPort(3232);

  ArduinoOTA.setHostname("ESP32-OTA");

  ArduinoOTA.setPassword("admin");

  ArduinoOTA
  .onStart([]() {
    String type;
    if (ArduinoOTA.getCommand() == U_FLASH)
      type = "sketch";
    else // U_SPIFFS
      type = "filesystem";

    // NOTE: if updating SPIFFS this would be the place to unmount SPIFFS using SPIFFS.end()
    Serial.println("Start updating " + type);
  })
  .onEnd([]() {
    Serial.println("\nEnd");
  })
  .onProgress([](unsigned int progress, unsigned int total) {
    Serial.printf("Progress: %u%%\r", (progress / (total / 100)));
  })
  .onError([](ota_error_t error) {
    Serial.printf("Error[%u]: ", error);
    if (error == OTA_AUTH_ERROR) Serial.println("Auth Failed");
    else if (error == OTA_BEGIN_ERROR) Serial.println("Begin Failed");
    else if (error == OTA_CONNECT_ERROR) Serial.println("Connect Failed");
    else if (error == OTA_RECEIVE_ERROR) Serial.println("Receive Failed");
    else if (error == OTA_END_ERROR) Serial.println("End Failed");
  });

  ArduinoOTA.begin();

  Serial.println("Ready");
  Serial.print("IP address: ");
  Serial.println(WiFi.localIP());
  lastRestartTime = millis();
}

void WiFiStationConnected(WiFiEvent_t event, WiFiEventInfo_t info) {
  leds.setPixelColor(0, leds.Color(52, 235, 229));
  leds.show();
  Serial.println("Connected to AP successfully!");
}

void WiFiGotIP(WiFiEvent_t event, WiFiEventInfo_t info) {
  leds.setPixelColor(0, leds.Color(52, 235, 229));
  leds.show();
  Serial.println("WiFi connected");
  Serial.println("IP address: ");
  Serial.println(WiFi.localIP());
}

void WiFiStationDisconnected(WiFiEvent_t event, WiFiEventInfo_t info) {
  leds.setPixelColor(0, leds.Color(255, 0, 0));
  leds.show();
  Serial.println("Disconnected from WiFi access point");
  Serial.print("WiFi lost connection. Reason: ");
  Serial.println(info.wifi_sta_disconnected.reason);
  Serial.println("Trying to Reconnect");
  WiFi.mode(WIFI_STA);
  WiFi.begin(ssid, password);
}

void loop() {
  ArduinoOTA.handle();

  buttonState = digitalRead(BUTTON_PIN);

  if (buttonState != lastButtonState) {
    if (buttonState == LOW) {
      buttonClicks++;
      Serial.print("Button pressed ");
      Serial.println(buttonClicks);
      buttonTimer = millis(); // Update the button timer every time the button is pressed
      if (buttonClicks == 2) {
        buttonClicks = 0;
        leds.setPixelColor(0, leds.Color(0, 0, 255));
        leds.show();
        // Rest of the code for sending the message
        HTTPClient http;

        http.begin(apihost);
        http.addHeader("Authorization", String("Bearer ") + bearerToken);
        http.addHeader("Content-Type", "application/json");

        String payload = "{\"messageTemplateId\":\"<messageTemplateId of the message you want to send>\",\"subject\":\"<The Subject of the Message, Needs to be the subject of the messageId>\",\"distributionListIds\":\"<Insert the ID of the Distribution>\"}";

        int httpCode = http.POST(payload);
        if (httpCode > 0) {
          Serial.println("Request sent successfully");
          Serial.println(httpCode);
          Serial.println(http.getString());
        } else {
          Serial.println("Request failed with status code:");
          Serial.println(httpCode);
        }
        http.end();
      }
      else {
        leds.setPixelColor(0, leds.Color(0, 30, 0));
        leds.show();
      }
    }
    lastButtonState = buttonState;
  }

  // Check if the timeout has passed without another click
  if (buttonClicks == 1 && (millis() - buttonTimer) >= BUTTON_TIMEOUT) {
    buttonClicks = 0;
    leds.setPixelColor(0, leds.Color(0, 30, 0));
    leds.show();
  }

  if (buttonClicks == 0 && (millis() - buttonTimer) >= BUTTON_DELAY) {
    leds.setPixelColor(0, leds.Color(0, 30, 0));
    leds.show();

//86400000
    if (millis() - lastRestartTime >= 86400000) {
      ESP.restart();
    }
  }
}
